# Cognite REVEAL 3D Web Viewer

> Visualize Cognite's 3D models in a web browser with [WebGL](https://en.wikipedia.org/wiki/WebGL).

_Note: The component will by default send anonymous usage statistics. This is used to improve the 3D viewer. You can opt out from this in the Cognite3DViewer constructor._

## Table of Contents

- [Install](#install)
- [Cognite Data Fusion](#cognite-data-fusion)
- [Usage](#usage)
  - [Sample code](#sample-code)
    - [Add progress listeners](#add-progress-listeners)
    - [Make the model clickable](#make-the-model-clickable)
- [API Reference](#api-reference)
- [Support](#support)

## Prerequisites

You need to install the [Cognite SDK](https://www.npmjs.com/package/@cognite/sdk) which needs to be authenticated before loading the 3D viewer.

## Install

Install the package with yarn:

    yarn add @cognite/3d-viewer

or npm:

    npm install @cognite/3d-viewer

## Cognite Data Fusion

To be able to use this 3D viewer you need to have access to Cognite Data Fusion. You can upload 3D models using [Cognite 3D ingestor](http://3d-ingestor.cogniteapp.com), or by looking at [Cognite's API documentation](https://doc.cognitedata.com/api/0.6/#tag/3D). You can then use this viewer to visualize the 3D model in any web-based application on desktop or mobile devices.

## Usage

```js
import { Cognite3DViewer, THREE, TWEEN } from '@cognite/3d-viewer';
//  If you are loading a local file, then just pass it in the localPath when you addModel:
const viewer = new Cognite3DViewer();
// If you are loading more than just local file, you would need to specify a sdk:
const viewer = new Cognite3DViewer({ sdk: new CogniteClient({ ... }) })
// Create three.js objects
const vector = new THREE.Vector3();
// Use TWEEN to animate e.g. the camera, see npmjs.com/package/@tweenjs/tween.js
```

### Requirements

To be able to use this package you need the following polyfills:

- [babel-polyfill](https://babeljs.io/docs/en/babel-polyfill)

### Sample code

Adding model from CDF

```js
// The viewer will render to a canvas inside a wrapper container.
// You can specify the wrapper by setting options.domElement and passing the options argument
// to the Cognite3DViewer constructor. If not set, it will create a div element for you.
const viewer = new Cognite3DViewer({ sdk: new CogniteClient({ ... }) });

// You can then add the div to your existing DOM:
document.body.appendChild(viewer.domElement);

// At this point you will only see a black canvas.

// So let's add a 3D model:
const options = {
  modelId, // 3D model id
  revisionId, // The model's revision id
};
viewer.addModel(options).then(function(model) {
  // Move camera to look at the model
  viewer.fitCameraToModel(model, 0);
});
```

Adding local model

```js
// The viewer will render to a canvas inside a wrapper container.
// You can specify the wrapper by setting options.domElement and passing the options argument
// to the Cognite3DViewer constructor. If not set, it will create a div element for you.
const viewer = new Cognite3DViewer();

// You can then add the div to your existing DOM:
document.body.appendChild(viewer.domElement);

// At this point you will only see a black canvas.

// So let's add a 3D model:
const options = {
  localPath: '...',
};
viewer.addModel(options).then(function(model) {
  // Move camera to look at the model
  viewer.fitCameraToModel(model, 0);
});
```

> Note: You may need additional styling to the wrapper div container to make it fit your use-case.
> By default, the wrapper will have this style:
>
> ```css
>  {
>   width: 100vw;
>   height: 100vh;
> }
> ```
>
> The internal canvas will be styled to fill this wrapper.

#### Add progress listeners

```js
function onProgress(progress) {
  console.log(progress);
}

function onComplete() {
  console.log('Model loaded');
}

const options = {
  modelId,
  revisionId,
  onProgress, // optional
  onComplete, // optional
};
viewer.addModel(options)...
```

#### Make the viewer clickable

- **See: [getBoundingBox](#getboundingbox)**
- **See: [getIntersectionFromPixel](#getintersectionfrompixel)**
- **See: [fitCameraToBoundingBox](#fitcameratoboundingbox)**
- **See: [deselectAllNodes](#deselectallnodes)**

```js
viewer.on('click', function(event) {
  const { offsetX, offsetY } = event;
  const intersection = viewer.getIntersectionFromPixel(offsetX, offsetY);
  if (intersection !== null) {
    const { nodeId, point, model } = intersection;
    console.log('User clicked at world coordinate', point);
    // highlight the object
    model.selectNode(nodeId);
    // make the camera zoom to the object
    const boundingBox = model.getBoundingBox(nodeId);
    viewer.fitCameraToBoundingBox(boundingBox, 2000); // 2 sec
  } else {
    // Clicked outside the 3D model
    model.deselectAllNodes();
  }
});
```

#### Load saved camera position from the API

Assume you have the revision object from Cognite Data Fusion which you can get from this [endpoint](https://doc.cognitedata.com/api/0.6/#operation/getRevision).

Here is a code snippet to use the saved camera position:

```js
const { target, position } = revision.camera;
if (Array.isArray(target) && Array.isArray(position)) {
  // Create three.js objects
  const positionVector = new THREE.Vector3(...position);
  const targetVector = new THREE.Vector3(...target);
  // Apply transformation matrix
  positionVector.applyMatrix4(model.matrix);
  targetVector.applyMatrix4(model.matrix);
  // Set on viewer
  viewer.setCameraPosition(positionVector);
  viewer.setCameraTarget(targetVector);
} else {
  viewer.fitCameraToModel(model, 0);
}
```

## API Reference

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

- [Cognite3DViewer](#cognite3dviewer)
  - [Parameters](#parameters)
  - [Examples](#examples)
  - [dispose](#dispose)
  - [domElement](#domelement)
  - [isBrowserSupported](#isbrowsersupported)
  - [on](#on)
  - [off](#off)
  - [addModel](#addmodel)
  - [addObject3D](#addobject3d)
  - [removeObject3D](#removeobject3d)
  - [setSlicingPlanes](#setslicingplanes)
  - [getCameraPosition](#getcameraposition)
  - [getCameraTarget](#getcameratarget)
  - [setCameraPosition](#setcameraposition)
  - [setCameraTarget](#setcameratarget)
  - [fitCameraToBoundingBox](#fitcameratoboundingbox)
  - [disableKeyboardNavigation](#disablekeyboardnavigation)
  - [enableKeyboardNavigation](#enablekeyboardnavigation)
  - [fitCameraToModel](#fitcameratomodel)
  - [worldToScreen](#worldtoscreen)
  - [getIntersectionFromPixel](#getintersectionfrompixel)
  - [getScreenshot](#getscreenshot)
- [Intersection](#intersection)
- [Cognite3DModel](#cognite3dmodel)
  - [getSubtreeNodeIds](#getsubtreenodeids)
  - [getBoundingBox](#getboundingbox)
  - [iterateNodes](#iterateNodes)
  - [iterateSubtree](#iterateSubtree)
  - [getNodeColor](#getnodecolor)
  - [setNodeColor](#setnodecolor)
  - [resetNodeColor](#resetnodecolor)
  - [selectNode](#selectnode)
  - [deselectNode](#deselectnode)
  - [deselectAllNodes](#deselectallnodes)
  - [showNode](#shownode)
  - [showAllNodes](#showallnodes)
  - [hideAllNodes](#hideallnodes)
  - [hideNode](#hidenode)

### Cognite3DViewer

`Cognite3DViewer` is the root class of a Cognite 3D viewer. It controls all aspects of the 3D viewer.

#### Parameters

- `options` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** (optional, default `{}`)
  - `options.domElement` **[Element](https://developer.mozilla.org/docs/Web/API/Element)?** \-- An existing DOM element that we will render into. This corresponds to the domElement property below.
  - `options.noBackground` **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)?** \-- Transparent background or not (optional, default `false`)
  - `options.logMetrics` **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** \-- Send anonymous usage statistics. (optional, default `true`)
  - `options.highlightColor` **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** \-- Highlight color of the selected objects (optional, default `THREE.Color(0,0,1)`)
  - `options.viewCube` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** \-- If defined then show a view cube and snap location of the view cube to this value. One of: 'topleft', 'topright', 'bottomleft', 'bottomright'.

#### Examples

```javascript
const viewer = new Cognite3DViewer({
  noBackground: true,
  sdk: <Instance of CogniteClient>
});
```

#### dispose

- **See: <https://threejs.org/docs/#manual/en/introduction/How-to-dispose-of-objects>**

Dispose of WebGL resources. Can be used to free up memory when the viewer is no longer in use.

##### Examples

```javascript
// Viewer is no longer in use, free up memory
viewer.dispose();
```

#### domElement

The DOM element the viewer will insert its rendering canvas into.
The DOM element can be specified in the options when the viewer is created.
If not specified, the DOM element will be created automatically.
The DOM element cannot be changed after the viewer has been created.

#### isBrowserSupported

Check if the viewer supports the current browser

##### Examples

```javascript
if (!Cognite3DViewer.isBrowserSupported()) {
  // create an error message to the user?
}
```

#### on

- **See: [off](#off)**

Add event listener to the viewer
call [off](#off) to remove an event listener

##### Parameters

- `type` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** \-- Event type ('click' or 'cameraChange')
- `func` **function (event:PointerEvent)** \-- The callback function

##### Examples

```javascript
const onClick = event => { ... };
viewer.on('click', onClick);
```

```javascript
viewer.on('cameraChange', (position, target) => {
  console.log('Camera changed: ', position, target);
});
```

#### off

- **See: [on](#on)**

Remove event listener from the viewer
call [on](#on) to add an event listener

##### Parameters

- `type` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** \-- Event type ('click' or 'cameraChange')
- `func` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** \-- The callback function used in [on](#on)

##### Examples

```javascript
viewer.off('click', onClick);
```

#### addModel

Add a new 3D model to the viewer. Call [fitCameraToModel](#fitcameratomodel) to focus camera on the model after it has loaded.

##### Parameters

- `options` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** (optional, default `{}`)
  - `options.modelId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The model's id
  - `options.revisionId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The model's revision id
  - `options.geometryFilter` { boundingBox?: **THREE.Box3** } \-- Filter out geometries. We currently only support bounding box filter, i.e. load only geometries inside the given bounding box.
  - `options.onProgress` **function (progress: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object))?** \-- Callback for progress events
  - `options.onComplete` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)?** \-- Callback when the model is fully loaded

##### Examples

```javascript
// Load a model and focus camera on the model
const options = {
  modelId: 'COGNITE_3D_MODEL_ID',
  revisionId: 'COGNITE_3D_REVISION_ID',
};
viewer.addModel(options).then(model => {
  viewer.fitCameraToModel(model, 0);
});
```

```javascript
// Load only geometries inside a bounding box, then focus camera on that area
const boundingBox = new THREE.Box3(
  new THREE.Vector3(0, 0, 0),
  new THREE.Vector3(10, 10, 10)
);

const options = {
  modelId: 'COGNITE_3D_MODEL_ID',
  revisionId: 'COGNITE_3D_REVISION_ID',
  geometryFilter: { boundingBox },
};

viewer.addModel(options).then(model => {
  // Transform the bounding box to model space
  boundingBox.applyMatrix4(model.matrix);
  // Focus camera on bounding box
  viewer.fitCameraToBoundingBox(boundingBox);
});
```

Returns **[Promise](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;[Cognite3DModel](#cognite3dmodel)>**

#### addObject3D

Add a THREE.Object3D to the viewer

##### Parameters

- `object` **THREE.Object3D** \-- A three.js object

##### Examples

```javascript
const sphere = new THREE.Mesh(
  new THREE.SphereBufferGeometry(),
  new THREE.MeshBasicMaterial()
);
viewer.addObject3D(sphere);
```

#### removeObject3D

Remove a THREE.Object3D from the viewer

##### Parameters

- `object` **THREE.Object3D** \-- A three.js object

##### Examples

```javascript
const sphere = new THREE.Mesh(
  new THREE.SphereBufferGeometry(),
  new THREE.MeshBasicMaterial()
);
viewer.addObject3D(sphere);
viewer.removeObject3D(sphere);
```

#### setSlicingPlanes

Sets per-pixel slicing planes. Pixels behind any of the planes will be sliced away.

##### Parameters

- `slicingPlanes` **THREE.Plane[]** \-- The planes to use for slicing

##### Examples

```javascript
// Hide pixels with values less than 0 in the x direction
const plane = new THREE.Plane(new THREE.Vector3(1, 0, 0), 0);
viewer.setSlicingPlanes([plane]);
```

```javascript
// Hide pixels with values greater than 20 in the x direction
const plane = new THREE.Plane(new THREE.Vector3(-1, 0, 0), 20);
viewer.setSlicingPlanes([plane]);
```

```javascript
// Hide pixels with values less than 0 in the x direction or greater than 0 in the y direction
const xPlane = new THREE.Plane(new THREE.Vector3(1, 0, 0), 0);
const yPlane = new THREE.Plane(new THREE.Vector3(0, -1, 0), 0);
viewer.setSlicingPlanes([xPlane, yPlane]);
```

```javascript
// Hide pixels behind an arbitrary, non axis-aligned plane
const plane = new THREE.Plane(new THREE.Vector3(1.5, 20, -19), 20);
viewer.setSlicingPlanes([plane]);
```

```javascript
// Disable slicing planes
viewer.setSlicingPlanes([]);
```

#### getCameraPosition

Returns camera's position

##### Examples

```javascript
const position = viewer.getCameraPosition();
```

Returns **THREE.Vector3** Position in world space

#### getCameraTarget

Returns camera's target

##### Examples

```javascript
const target = viewer.getCameraTarget();
```

Returns **THREE.Vector3** Target in world space

#### setCameraPosition

Set camera's position

##### Parameters

- `position` **THREE.Vector3** \-- Position in world space

##### Examples

```javascript
// store position, target
const position = viewer.getCameraPosition();
const target = viewer.getCameraTarget();
// restore position, target
viewer.setCameraPosition(position);
viewer.setCameraTarget(target);
```

#### setCameraTarget

Set camera's target

##### Parameters

- `target` **THREE.Vector3** \-- Target in world space

##### Examples

```javascript
// store position, target
const position = viewer.getCameraPosition();
const target = viewer.getCameraTarget();
// restore position, target
viewer.setCameraPosition(position);
viewer.setCameraTarget(target);
```

#### fitCameraToBoundingBox

- **See: [getBoundingBox](#getboundingbox)**
  Move camera to a place where the content of a bounding box is visible to the camera.

##### Parameters

- `box` **THREE.Box3** \-- The bounding box in world space
- `duration` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)?** \-- The duration of the animation moving the camera. Set this to 0 (zero) to disable animation.
- `radiusFactor` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)?** \-- The ratio of the distance from camera to center of box and radius of the box (optional, default `4`)

##### Examples

```javascript
// Focus camera on bounding box over 500 milliseconds
viewer.fitCameraToBoundingBox(boundingBox, 500);
```

```javascript
// Focus camera on bounding box instantaneously
viewer.fitCameraToBoundingBox(boundingBox, 0);
```

```javascript
// Place the camera closer to the bounding box
viewer.fitCameraToBoundingBox(boundingBox, 500, 2);
```

```javascript
// Animate camera to focus on a specific object with a given node id
const nodeId = 1234;
const boundingBox = model.getBoundingBox(nodeId);
viewer.fitCameraToBoundingBox(boundingBox, 1000);
```

#### disableKeyboardNavigation

- **See: [enableKeyboardNavigation](#enablekeyboardnavigation)**

Disables camera navigation with the keyboard.

##### Examples

```javascript
// Disable keyboard navigation on textbox focus
inputElement.addEventListener('focus', () => {
  viewer.disableKeyboardNavigation();
});
```

#### enableKeyboardNavigation

- **See: [disableKeyboardNavigation](#disablekeyboardnavigation)**

Enables camera navigation with the keyboard.

##### Examples

```javascript
// Enable keyboard navigation on textbox blur
inputElement.addEventListener('blur', () => {
  viewer.enableKeyboardNavigation();
});
```

#### fitCameraToModel

Move camera to a place where the 3D model is visible. It uses the bounding box of the 3D model and calls [fitCameraToBoundingBox](#fitcameratoboundingbox)

##### Examples

```javascript
// Fit camera to model
viewer.fitCameraToModel(model);
```

```javascript
// Fit camera to model over 500 milliseconds
viewer.fitCameraToModel(model, 500);
```

```javascript
// Fit camera to model instantaneously
viewer.fitCameraToModel(model, 0);
```

##### Parameters

- `model` **[Cognite3DModel](#cognite3dmodel)** \-- The 3D model
- `duration` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)?** \-- Same as for [fitCameraToBoundingBox](#fitcameratoboundingbox)

#### worldToScreen

- **See: <https://www.w3schools.com/graphics/canvas_coordinates.asp>**

Convert a point in world space to its coordinates in the canvas. This can be used to place HTML objects near 3D objects on top of the 3D viewer.

##### Parameters

- `point` **THREE.Vector3** \-- World space coordinate.
- `normalize` **bool?** \-- Optional. If true, coordinates are normalized into [0,1]. If false, the values are in the range \[0, &lt;canvas_size>). (optional, default `false`)

##### Examples

```javascript
// Find canvas coordinates, i.e. where on the screen it is rendered, for an object with a given node id
const boundingBoxCenter = new THREE.Vector3();
// Find center of bounding box in world space
model.getBoundingBox(nodeId).getCenter(boundingBoxCenter);
// Screen coordinates of that point
const screenCoordinates = viewer.worldToScreen(boundingBoxCenter);
```

```javascript
// Find normalized canvas coordinates, i.e. where on the screen it is rendered, for an object with a given node id
const boundingBoxCenter = new THREE.Vector3();
// Find center of bounding box in world space
model.getBoundingBox(nodeId).getCenter(boundingBoxCenter);
// Screen coordinates of that point normalized in the range [0,1]
const screenCoordinates = viewer.worldToScreen(boundingBoxCenter, true);
```

```javascript
// Find canvas coordinates, i.e. where on the screen it is rendered, for an object with a given node id
const boundingBoxCenter = new THREE.Vector3();
// Find center of bounding box in world space
model.getBoundingBox(nodeId).getCenter(boundingBoxCenter);
// Screen coordinates of that point
const screenCoordinates = viewer.worldToScreen(boundingBoxCenter);
if (screenCoordinates == null) {
  // Object not visible on screen
} else {
  // Object is visible on screen
}
```

Returns **(THREE.Vector2 | null)** \-- Returns 2D coordinates if the point is visible on screen, or `null` if object is outside screen.

#### getIntersectionFromPixel

Raycasting model(s) for finding where the ray intersects with the model.

##### Parameters

- `x` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- X coordinate in pixels (relative to the domElement)
- `y` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- Y coordinate in pixels (relative to the domElement)
- `cognite3DModel` **[Cognite3DModel](#cognite3dmodel)?** \-- If specified then only give results for this model

##### Examples

```javascript
const intersection = viewer.getIntersectionFromPixel(50, 100); // x = 50 pixels from the left, y = 100 pixels from the top
if (intersection)
  // it was a hit
  console.log(
    'You hit model ',
    intersection.model,
    ' at the node with id ',
    intersection.nodeId,
    ' at this exact point ',
    intersection.point
  );
```

Returns **([Intersection](#intersection) | null)** \-- If there was an intersection then return the intersection object - otherwise it returns null if there was no intersections.

#### getScreenshot

Take screenshot from the current camera position.

##### Examples

```javascript
// Add screenshot with resolution of the canvas to the page
const url = await viewer.getScreenshot();
const image = document.createElement('img');
image.src = url;
document.body.appendChild(url);
```

```javascript
// Take screenshot with custom resolution
const url = await viewer.getScreenshot(1920, 1080);
```

##### Parameters

- `width` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)?** \-- Width of the final image. Default is current canvas size.
- `height` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)?** \-- Height of the final image. Default is current canvas size.

Returns **[Promise](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>** A [Blob URL](https://developer.mozilla.org/en-US/docs/Web/API/URL/createObjectURL) to the image ('image/png')

### Intersection

Type: [object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)

#### Properties

- `model` **[Cognite3DModel](#cognite3dmodel)** \-- The node id
- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The nodeId of the first intersected node
- `point` **THREE.Vector3** \-- The 3D position of the intersection point in the world coordinate system

### Cognite3DModel

**Extends THREE.Object3D**

`Cognite3DModel` is the class representing a Cognite 3D model and its state

#### modelId

The id of the model in Cognite Data Fusion

#### revisionId

The id of the model revision in Cognite Data Fusion

#### getSubtreeNodeIds

Get all node ids in a subtree where the root node has id 'nodeId'

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The ID of the root node
- `subtreeSize` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)?** \-- If you already know the subtreeSize, this will avoid an extra API call

Returns **\[[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)]** List of nodeIds in the subtree

#### getBoundingBox

Get bounding box of a node

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id
- `box` **THREE.Box3?** \-- Optional target. Specify this to increase performance if the box can be reused.

##### Examples

```javascript
const box = model.getBoundingBox(nodeId);
```

```javascript
const reusableBox = new THREE.Box3();
const box = model.getBoundingBox(nodeId, reusableBox);
// box === reusableBox
```

Returns **THREE.Box3**

#### iterateNodes

Go through the node tree and apply an action to each node that has a node ID.

##### Examples

```javascript
const url = await model.iterateNodes(console.log);
```

##### Parameters

- `action` **Callback** \-- Callback function taking in a nodeId and treeId

Returns **void**

#### iterateSubtree

Go through the subtree a node of the tree and apply an action to each subtree node that has a node ID.

##### Examples

```javascript
const url = await viewer.iterateSubtree(1111111111, console.log);
```

##### Parameters

- `nodeId` **number** \-- The id of the node whose subtree will have action applied to
- `action` **Callback** \-- Callback function taking in a nodeId and treeId
- `treeIndex?` **number** \-- (Optional) The index within the tree for the given nodeId
- `subtreeSize?` **number** \-- (Optional) The size of the subtree starting from nodeId

Returns **Promise&lt;boolean>** Whether a valid nodeId was given or not

#### getNodeColor

- **See: [setNodeColor](#setnodecolor)**

Return the color on a node.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id

Returns **{r: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number), g: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number), b: [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)}** (r, g, b in the range [0, 255])

#### setNodeColor

- **See: [getNodeColor](#getnodecolor)**

Set the color on a node.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id
- `r` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The red color value (0-255)
- `g` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The green color value (0-255)
- `b` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The blue color value (0-255)

#### resetNodeColor

Reset a node's color to its original color.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id

#### selectNode

- **See: [deselectNode](#deselectnode)**
- **See: [deselectAllNodes](#deselectallnodes)**

Mark a node as selected. Only visible nodes can be selected.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id

#### deselectNode

- **See: [selectNode](#selectnode)**
- **See: [deselectAllNodes](#deselectallnodes)**

Mark a node as deselected.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id

#### deselectAllNodes

Mark all selected nodes as deselected.

#### showNode

Show a node.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id

#### showAllNodes

Show all nodes.

#### hideAllNodes

Hide all nodes.

##### Parameters

- `ghost` **bool** \-- Hide with ghost effect (optional, default `false`)

#### hideNode

Hide a node.

##### Parameters

- `nodeId` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** \-- The node's id
- `ghost` **bool** \-- Hide with ghost effect (optional, default `false`)

## Support

For support contact <mailto:support@cognite.com>
